#!/usr/bin/env python3

DPI=300

def mm(x):
  return DPI*x/25.4

def pt(x):
  return DPI*x/72

PICWIDTH=mm(135)
PICHEIGHT=mm(45)

LINEWIDTH=pt(5)
LINEENDS=pt(5)
LINECOLOUR='black'

BOXMARGIN=pt(2)
BOXGAP=pt(5)
BOXCOLOUR=LINECOLOUR

PIPW=pt(2)
PIPH=pt(3)

YEARFONT = 'Liberation-Mono-Bold'
YEARSIZE = '10'
YEARCOLOUR='white'

ITEMFONT = 'DejaVu-Sans'
ITEMSIZE = '8'
OVERCOLOUR = BOXCOLOUR
UNDERCOLOUR = 'gray(50%)'

import sys, re
from subprocess import run


# Returns (height, ascent, advance)
def getmetrics(font, size):
  cmd=f"magick -debug annotate "+\
      f"xc: -font '{font}' -pointsize {size} -density {DPI} "+\
      "-annotate 0 '0000' null:"
  lines=run(cmd, shell=True, capture_output=True).stderr.decode().split('\n')
  mre=r'^ *Metrics: .*height: (\d*);.*ascent: (\d*);.*advance: (\d*);'
  m=list(filter(None, map(lambda s: re.match(mre, s), lines)))
  return tuple(map(int, m[0].groups()))

# Returns (width, height, lead, ascent)
def metrics(font, size, text):
  size=int(size)
  w=2*len(text)*DPI*size/72
  h=3*DPI*size/72
  x=DPI*size/72/2
  y=1.5*DPI*size/72
  cmd=f"magick -size {w}x{h} xc:none -fill black "+\
      f"-font '{font}' -pointsize {size} -density {DPI} "+\
      f"-annotate +{x}+{y} '{text}' -trim info:"
  infore=r'^xc:none XC (\d*)x(\d*) \d*x\d*\+(\d*)\+(\d*)'
  info=run(cmd, shell=True, capture_output=True).stdout.decode()
  width, height, xoff, yoff = tuple(map(int, re.match(infore, info).groups()))
  return (width, height, xoff-x, y-yoff)

if len(sys.argv)!=2:
  print()
  print('Bruk: %s <filnamn>'%sys.argv[0])
  print()
  print('Filens format är:')
  print()
  print('  1982 ZX Spectrum')
  print('  1984 ZX Spectrum+')
  print('  1985 ZX Spectrum 128')
  print('  1986 ZX Spectrum +2')
  print('  1987 ZX Spectrum +3')
  print()
  print('  1980 ZX80')
  print('  1981 ZX81')
  print('  1982 Commodore 64')
  print('  1984 Apple Macintosh')
  print('  1985 Atari ST')
  print()
  print('De övre raderna listar medlemmar av den familj datorn i fråga')
  print('tillhör, och visas ovanför tidslinjen. De nedre radera listar')
  print('andra datorer från samma period och visas under tidslinjen.')
  print('Rader inleds med årtal. Output skrivs till "tidslinje.png".')
  print()
  sys.exit(0)

try:
  lines=open(sys.argv[1], 'r').readlines()
except Exception as e:
  print(e)
  sys.exit(1)

try:
  i=lines.index('\n')
except:
  i=len(lines)
  print('Ingen tom rad i filen -> text endast ovanför tidslinjen.')

over=[(int(year), text) for (year, text) in 
      (l.strip().split(maxsplit=1) for l in lines[:i])]
under=[(int(year), text) for (year, text) in
      (l.strip().split(maxsplit=1) for l in lines[i+1:])]

start=min(year for (year, text) in over+under)
end=max(year for (year, text) in over+under)
years=list(range(start, end+1))
nyears=len(years)

over=dict(over)
under=dict(under)

yearw, yearh, yearlead, yearasc = metrics(YEARFONT, YEARSIZE, '1999')
_, _, _, itemasc = metrics(ITEMFONT, ITEMSIZE, 'XYZ')

boxw=yearw+2*BOXMARGIN
boxh=yearh+2*BOXMARGIN
boxtop=PICHEIGHT/2-boxh/2
boxbot=boxtop+boxh
yearbase=boxtop+yearasc+BOXMARGIN
linetop=PICHEIGHT/2-LINEWIDTH/2
linelen=nyears*boxw+(nyears-1)*BOXGAP+2*LINEENDS

boxline=['-fill', BOXCOLOUR]
yearline=['-pointsize', YEARSIZE, '-font', YEARFONT, '-fill', YEARCOLOUR]
overline=['-fill', OVERCOLOUR, '-font', ITEMFONT, '-pointsize', ITEMSIZE]
underline=['-fill', UNDERCOLOUR, '-font', ITEMFONT, '-pointsize', ITEMSIZE]

for n, year in enumerate(years):
  if year not in over and year not in under: continue
  x=n*(boxw+BOXGAP)+LINEENDS
  boxline+=['-draw', f'translate {x},{boxtop} rectangle 0,0 {boxw},{boxh}']
  yearline+=['-annotate', f'+{x+BOXMARGIN-yearlead}+{yearbase}', str(year)]
  if year in over:
    overline+=['-draw',
               f"translate {x+boxw/2},{boxtop-PIPH*1.2} "+\
               f"rotate -35 text 0,0 '{over[year]}'",
               '-draw',
               f"rectangle {x+(boxw-PIPW)/2},{boxtop-PIPH} "+\
                         f"{x+(boxw+PIPW)/2},{boxtop}"]
  if year in under:
    underline+=['-draw',
               f"translate {x+boxw/2},{boxbot+PIPH*1.2} "+\
               f"rotate +35 translate {0},{itemasc} text 0,0 '{under[year]}'",
               '-draw',
               f"rectangle {x+(boxw-PIPW)/2},{boxbot} "+\
                         f"{x+(boxw+PIPW)/2},{boxbot+PIPH}"]
line=['-stroke', 'none', '-fill', LINECOLOUR,
      '-draw', f'rectangle 0,{linetop} {linelen},{linetop+LINEWIDTH}']+\
     overline+underline+boxline+yearline

magick=['magick', '-size', f'{PICWIDTH}x{PICHEIGHT}', 'xc:white']+\
       ['-density', str(DPI)]+line+\
       ['tidslinje.png']
#print(' '.join(magick))
run(magick)


# ------------------------------

# -draw "translate 100,100 rotate -45 rectangle 0,-7 100,7 text 20,5 HELLO"
# -draw "circle 100,100 100,109"

#'DejaVu-Sans-Mono-Bold'
#'mnicmp-Bold'
#'TeleType-ASR-33-UK'
#'VT220'
#'Liberation-Mono-Bold'
